@@ -50,7 +50,7 @@ class RegressionTestMeta(type):
                 self._namespace.pop(key, None)
 
             elif key in self['_rfm_local_param_space']:
-                raise ValueError(
+                raise ReframeSyntaxError(
                     f'cannot override parameter {key!r}'
                 )
             else:
@@ -76,7 +76,7 @@ class RegressionTestMeta(type):
                 except KeyError:
                     # Handle parameter access
                     if key in self['_rfm_local_param_space']:
-                        raise ValueError(
+                        raise ReframeSyntaxError(
                             'accessing a test parameter from the class '
                             'body is disallowed'
                         ) from None
@@ -426,7 +426,7 @@ class RegressionTestMeta(type):
                     return
                 elif not var_space[name].field is value:
                     desc = '.'.join([cls.__qualname__, name])
-                    raise ValueError(
+                    raise ReframeSyntaxError(
                         f'cannot override variable descriptor {desc!r}'
                     )
 
@@ -437,7 +437,7 @@ class RegressionTestMeta(type):
         try:
             param_space = super().__getattribute__('_rfm_param_space')
             if name in param_space.params:
-                raise ValueError(f'cannot override parameter {name!r}')
+                raise ReframeSyntaxError(f'cannot override parameter {name!r}')
 
         except AttributeError:
             pass
